/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.eclipse.ui.IEditorPart;

/**
 * <p>IMultiPageTaxonEditorPage interface.</p>
 *
 * @author n.hoffmann
 * @created Jan 5, 2010
 * @version 1.0
 */
public interface IMultiPageTaxonEditorPage extends IEditorPart {

	/**
	 * Redraws this editor
	 * return true on success
	 *
	 * @return a boolean.
	 */
	public boolean redraw();
	
	/**
	 * Redraws the editor controls
	 *
	 * @param focus if set to true the focus is set on the editor
	 * @return true on success
	 */
	public boolean redraw(boolean focus);
	
	/**
	 * Whether the editor page is currently redrawing
	 *
	 * @return a boolean.
	 */
	public boolean isRedrawing();

	/**
	 * Sets the editor page into enabled or disabled state depending on the 
	 * value of the supplied parameter <code>enabled</code>.
	 * 
	 * @param enabled
	 */
	public void setEnabled(boolean enabled);

	/**
	 * Sets the editor page into disabled state and changes the 
	 * appearance accordingly. 
	 */
	void setDisabled();
	
	/**
	 * Sets the editor page into disabled state and changes the 
	 * appearance in order to indicate that an error has occurred.  
	 */
	public void setOnError();
	
}
