package eu.etaxonomy.taxeditor.editor.handler;


import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeNode;

import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;



/**
 * Tests types of specimens to add items to the context menu.
 *
 * @author pplitzner
 * @author BenStoever
 */
public class SpecimenPropertyTester extends PropertyTester {
    private static final String SEQUENCE = "isSequence"; //$NON-NLS-1$
    private static final String IS_SPECIMEN_OR_OBSERVATION_BASE = "isSpecimenOrObservationBase"; //$NON-NLS-1$
    private static final String IS_ADD_DERIVATE_ALLOWED = "isAddDerivateAllowed"; //$NON-NLS-1$
    private static final String IS_ADD_MEDIA_ALLOWED = "isAddMediaAllowed"; //$NON-NLS-1$


    public SpecimenPropertyTester() {}

    /** {@inheritDoc} */
    @Override
    public boolean test(Object receiver, String property, Object[] args, Object expectedValue) {
        if (receiver instanceof IStructuredSelection) {
            IStructuredSelection selection = (IStructuredSelection) receiver;
            TreeNode treeNodeOfSelection = EditorUtil.getTreeNodeOfSelection(selection);
            if (treeNodeOfSelection!=null) {
                if (SEQUENCE.equals(property)) {
                    return isSequence(treeNodeOfSelection.getValue());
                }
                else if (IS_SPECIMEN_OR_OBSERVATION_BASE.equals(property)) {
                    return isSpecimenOrObservationBase(treeNodeOfSelection.getValue());
                }
                else if (IS_ADD_DERIVATE_ALLOWED.equals(property)) {
                    return isAddDerivateAllowed(treeNodeOfSelection.getValue());
                }
                else if (IS_ADD_MEDIA_ALLOWED.equals(property)) {
                    return isAddMediaAllowed(treeNodeOfSelection.getValue());
                }
            }
        }
        return false;
    }


    private boolean isSpecimenOrObservationBase(Object object) {
    	return (object instanceof SpecimenOrObservationBase);
	}

    private boolean isAddDerivateAllowed(Object object) {
        return (object instanceof SpecimenOrObservationBase || object instanceof Sequence)
                && !(object instanceof MediaSpecimen);
    }

    private boolean isAddMediaAllowed(Object object) {
        return object instanceof SpecimenOrObservationBase
                && !(object instanceof MediaSpecimen);
    }

    private boolean isSequence(Object object) {
        return (object instanceof Sequence);
    }

}
