/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.key;

import java.util.Vector;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeEvent;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeEvent.EventType;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorInput;
import eu.etaxonomy.taxeditor.model.AbstractDataChangeBehaviour;

/**
 * @author n.hoffmann
 * @created May 5, 2011
 * @version 1.0
 */
public class KeyEditorDataChangeBehaviour extends AbstractDataChangeBehaviour {

	private final KeyEditor source;
	private PolytomousKeyEditorInput input;

	/**
	 * @param keyEditor
	 */
	public KeyEditorDataChangeBehaviour(KeyEditor keyEditor) {
		source = keyEditor;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.AbstractDataChangeBehaviour#reactOnDataChange(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void reactOnDataChange(CdmDataChangeMap changeEvents) {
		input = (PolytomousKeyEditorInput) source.getEditorInput();
		
		if(changeEvents.sizeByEventType(EventType.UPDATE) > 0){
			reactOnUpdate(changeEvents.getEvents(EventType.UPDATE));
		}
	}

	/** {@inheritDoc} */
	@Override
	public void reactOnUpdate(Vector<CdmDataChangeEvent> events){

		for(CdmDataChangeEvent event : events){
			
			// update editor title
			if(event.getEntity() instanceof PolytomousKey){
				if(input.getKey().equals(event.getEntity())){
					// set the name of the editor window
					source.setPartName();
				}
			}
		}
	}
}
