/**
 *
 */
package eu.etaxonomy.taxeditor.editor.key.polytomous;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IPolytomousKeyService;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.taxeditor.editor.key.AbstractIdentificationEditorInput;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 *
 */
public class PolytomousKeyEditorInput extends AbstractIdentificationEditorInput<PolytomousKey> {

    private final String name;
    private final UUID keyUuid;
    private PolytomousKey key;


    protected PolytomousKeyEditorInput(ConversationHolder conversation,
            UUID polytomousKeyUuid) {
        super(conversation);
        this.keyUuid = polytomousKeyUuid;
        List<String> propertyPath = new ArrayList<>();
        propertyPath.add("root");
        propertyPath.add("root.*");
        this.key = CdmStore.getService(IPolytomousKeyService.class).load(polytomousKeyUuid, propertyPath);
        this.name = key.getTitleCache();
    }

//    public PolytomousKeyEditorInput(UUID polytomousKeyUuid, String name) {
//        super(CdmStore.createConversation());
//        this.keyUuid = polytomousKeyUuid;
//        this.name = name;
//    }


    public static PolytomousKeyEditorInput NewInstance(UUID polytomousKeyUuid) throws Exception{
        try{
            ConversationHolder conversation = CdmStore.createConversation();
            return new PolytomousKeyEditorInput(conversation, polytomousKeyUuid);
        }catch(Exception e){
            throw e;
        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getName()
     */
    @Override
    public String getName() {
        return name;
    }

    @Override
    public PolytomousKey getKey() {
        return key;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object object) {
        if (object instanceof PolytomousKeyEditorInput
                && getKey() != null
                && getKey().equals(((PolytomousKeyEditorInput) object).getKey())
                ){
            return true;
        }
        return super.equals(object);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getRootEntities()
     */
    @Override
    public  List<PolytomousKey> getRootEntities() {
        return Arrays.asList(key);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {
       key = CdmStore.getService(IPolytomousKeyService.class).merge(key,true).getMergedEntity();
      
    }

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        Map<Object,List<String>> propertyPathsMap = new HashMap<Object,List<String>>();

        List<String> polytomousKeyNodePropertyPaths = Arrays.asList(new String[] {
                "statement" //$NON-NLS-1$
        });
         propertyPathsMap.put("children", polytomousKeyNodePropertyPaths); //$NON-NLS-1$
         return propertyPathsMap;
    }

}
