/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.zest.core.viewers.IGraphContentProvider;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.PolytomousKeyRelationship;

/**
 *
 * @author n.hoffmann
 * @created Mar 30, 2011
 * @version 1.0
 */
class PolytomousKeyGraphContentProvider implements IGraphContentProvider {

	private List<PolytomousKeyRelationship> relations;

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

	}

	@Override
	public void dispose() {

	}

	@Override
	public Object[] getElements(Object inputElement) {
		relations = new ArrayList<PolytomousKeyRelationship>();

		if (inputElement instanceof PolytomousKey) {
			getTreeRecursively(relations, inputElement);
		}
		return relations.toArray();
	}

	private void getTreeRecursively(List<PolytomousKeyRelationship> relations,
			Object parent) {
		List<PolytomousKeyNode> children;

		if (parent instanceof PolytomousKeyNode) {
			((PolytomousKeyNode) parent).removeNullValueFromChildren();
			children = ((PolytomousKeyNode) parent).getChildren();
		} else if (parent instanceof PolytomousKey) {
			children = new ArrayList<PolytomousKeyNode>();
			children.add(((PolytomousKey) parent).getRoot());
		} else {
			throw new RuntimeException(
					Messages.PolytomousKeyGraphContentProvider_WRONG_PARENT
							+ parent.getClass());
		}

		for (PolytomousKeyNode childNode : children) {
			relations.add(new PolytomousKeyRelationship(parent, childNode));
			getTreeRecursively(relations, childNode);
		}
	}

	@Override
	public Object getSource(Object relationship) {
		return ((PolytomousKeyRelationship) relationship).getSource();
	}

	@Override
	public Object getDestination(Object relationship) {
		return ((PolytomousKeyRelationship) relationship).getDestination();
	}

}
