/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.key.polytomous.handler;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.key.KeyEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.IPolytomousKeyEditorPage;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorLabels;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyListEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.InsertPolytomousKeyNodeOperation;

/**
 * @author k.luther
 * @date 17.11.2016
 *
 */
public class InsertNewNodeHandler extends AbstractPolytomousKeyNodeHandler{

    private PolytomousKeyNode parentNode ;


    public InsertNewNodeHandler() {
        super(PolytomousKeyEditorLabels.INSERT_NODE_POLYTOMOUS_KEY_NODE_LABEL);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        IEditorPart editor = HandlerUtil.getActiveEditor(event);

        if (editor instanceof KeyEditor) {
            editorPage = (IPolytomousKeyEditorPage) ((KeyEditor) editor)
                    .getActiveEditor();

            if (editorPage instanceof PolytomousKeyListEditor) {
                PolytomousKeyListEditor klEditor = (PolytomousKeyListEditor) editorPage;
                if(klEditor.getTableItemCount() == 0) {
                    PolytomousKey pk = klEditor.getViewerInputKey();
                    parentNode = pk.getRoot();

                } else {
                    IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getCurrentSelection(event);
                    if (selection.getFirstElement() instanceof PolytomousKeyNode) {
                        parentNode = ((PolytomousKeyNode) selection.getFirstElement()).getParent();
                    } else {
                        return new Status(IStatus.ERROR,
                                "unknown", //$NON-NLS-1$
                                PolytomousKeyEditorLabels.NO_KEY_NODE_FOR_INSERT_NODE_SELECTED_MESSAGE);
                    }
                }
            }
        }
        return Status.OK_STATUS;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        return new InsertPolytomousKeyNodeOperation(event.getTrigger(), false, parentNode, editorPage);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void onComplete() {
        // TODO Auto-generated method stub

    }

}
