/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.key.polytomous.handler;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.key.KeyEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.IPolytomousKeyEditorPage;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorLabels;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyListEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.RemotingCreatePolytomousKeyNodeOperation;

/**
 * @author cmathew
 * @date 21 Jul 2015
 *
 */
public class RemotingCreateChildPolytomousKeyNodeHandler extends AbstractPolytomousKeyNodeHandler {

    private PolytomousKeyNode parentNode;
    IPolytomousKeyEditorPage editorPage;


    public RemotingCreateChildPolytomousKeyNodeHandler() {
        super(PolytomousKeyEditorLabels.CREATE_CHILD_POLYTOMOUS_KEY_NODE_LABEL);
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#allowOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        IEditorPart editor = HandlerUtil.getActiveEditor(event);

        if (editor instanceof KeyEditor) {
            editorPage = (IPolytomousKeyEditorPage) ((KeyEditor) editor)
                    .getActiveEditor();

            if (editorPage instanceof PolytomousKeyListEditor) {
                PolytomousKeyListEditor klEditor = (PolytomousKeyListEditor) editorPage;
                if(klEditor.getTableItemCount() == 0) {
                    PolytomousKey pk = klEditor.getViewerInputKey();
                    parentNode = pk.getRoot();

                } else {
                    IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getCurrentSelection(event);
                    if (selection.getFirstElement() instanceof PolytomousKeyNode) {
                        parentNode = (PolytomousKeyNode) selection.getFirstElement();
                    } else {
                        return new Status(IStatus.ERROR,
                                "unknown", //$NON-NLS-1$
                                PolytomousKeyEditorLabels.NO_KEY_NODE_FOR_INSERT_NODE_SELECTED_MESSAGE);
                    }
                }
            }
        }
        return Status.OK_STATUS;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#prepareOperation(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        return new RemotingCreatePolytomousKeyNodeOperation(event.getTrigger(), false, parentNode, editorPage);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#onComplete()
     */
    @Override
    public void onComplete() {


    }



}
