/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.key.polytomous.operation;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

import eu.etaxonomy.cdm.api.application.CdmChangeEvent.Action;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operation.RemotingCdmDefaultOperation;

/**
 * @author k.luther
 * @date 17.11.2016
 *
 */
public class InsertPolytomousKeyNodeOperation extends RemotingCdmDefaultOperation  {

    private final PolytomousKeyNode parentNode;

    private final static String LABEL = Messages.InsertPolytomousKeyNodeOperation_INSERT_KEY;

    /**
    *
    */
   protected IPostOperationEnabled postOperationEnabled;

    /**
     * @param label
     * @param action
     * @param source
     * @param async
     */
    public InsertPolytomousKeyNodeOperation(Object source,
            boolean async,
            PolytomousKeyNode parentNode, IPostOperationEnabled postOperationEnabled) {
        super(LABEL, Action.Create, source, async);
        this.parentNode = parentNode;
        this.postOperationEnabled = postOperationEnabled;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected CdmBase doSimpleExecute(IProgressMonitor monitor, IAdaptable info) throws Exception {
        PolytomousKeyNode childNode = PolytomousKeyNode.NewInstance();
        List<PolytomousKeyNode> newChildren = parentNode.getChildren();
        List<PolytomousKeyNode> copy = new ArrayList<PolytomousKeyNode>();
        for (PolytomousKeyNode node: newChildren) {
            copy.add(node);
        }
        parentNode.addChild(childNode, 0);
        for (PolytomousKeyNode node: copy){
           childNode.addChild(node);
        }
        this.postOperationEnabled.postOperation(childNode);
        return childNode;
    }

}
