/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @created Jan 20, 2011
 * @version 1.0
 */
public class AcceptedGroup extends AbstractHomotypicalGroupContainer {

	private AcceptedNameContainer acceptedContainer;

	/**
	 * @param editor
	 * @param group
	 */
	public AcceptedGroup(TaxonNameEditor editor, HomotypicalGroup group) {
		super(editor, group);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroup#dragEntered()
	 */
	@Override
	public void dragEntered() {
		Color color = EditorUtil.getColor(Resources.COLOR_DRAG_ENTER);
		acceptedContainer.setBackground(color);
		super.dragEntered();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroup#dragLeft()
	 */
	@Override
	public void dragLeft() {
		acceptedContainer.restoreColor();
		super.dragLeft();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.container.AbstractGroup#getGroupedContainers()
	 */
	@Override
	public List<AbstractGroupedContainer> getGroupedContainers() {
		List<AbstractGroupedContainer> containers = new ArrayList<AbstractGroupedContainer>();
		containers.add(getAcceptedNameContainer());
		containers.addAll(super.getGroupedContainers());
		return containers;
	}
	
	/**
	 * 
	 */
	protected void createContainers() {
		this.acceptedContainer = new AcceptedNameContainer(getEditor().getTaxon());
		acceptedContainer.setGroup(this);
		
		acceptedContainer.createContent();
		
		createSynonymContainer();
	}
	
	protected void createSynonymContainer(){
		for(Synonym synonym : getSynonyms()){
			SynonymContainer container = new SynonymContainer(this, synonym);
			this.add(container);
			container.createContent();
		}
	}

	/**
	 * @param acceptedNameContainer
	 */
	public void setAcceptedNameContainer(AcceptedNameContainer acceptedNameContainer) {
		this.acceptedContainer = acceptedNameContainer;
		acceptedContainer.setGroup(this);
	}
	
	public AcceptedNameContainer getAcceptedNameContainer(){
		return acceptedContainer;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroup#emptyGroup()
	 */
	@Override
	protected void emptyGroup() {
		acceptedContainer.dispose();
		acceptedContainer = null;
		super.emptyGroup();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractHomotypicalGroupContainer#redrawNeeded(eu.etaxonomy.cdm.model.name.HomotypicalGroup)
	 */
	@Override
	protected boolean redrawNeeded(HomotypicalGroup homotypicalGroup) {
		if(! acceptedContainer.getData().equals(getEditor().getTaxon())){
			return true;
		}
		return super.redrawNeeded(homotypicalGroup);
	}
}
