/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import javax.persistence.GenerationType;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * <p>MisapplicationContainer class.</p>
 *
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class MisapplicationContainer extends AbstractGroupedContainer<Taxon> {
	TaxonRelationship misapplication;

	/**
	 * <p>Constructor for MisapplicationContainer.</p>
	 *
	 * @param editor a {@link eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor} object.
	 * @param group a {@link eu.etaxonomy.taxeditor.editor.name.container.AbstractGroup} object.
	 * @param misappliedName a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 */
	public MisapplicationContainer(AbstractGroup group, Taxon misappliedName) {
		super(misappliedName);
		
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	/** {@inheritDoc} */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setFont(getViewerFont());
		TaxonNameEditor editor = getEditor();
		Taxon taxon = editor.getTaxon();
		for (TaxonRelationship rel: taxon.getTaxonRelations(getMisappliedName())){
			if (rel.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR())){
				misapplication = rel;
				break;
			}
		}
		showSec();

		initTextViewer();
	}

	@Override
	protected void updateIcon() {
		setIcon(MISAPPLIEDNAME_ICON);
	}

	private void showSec() {
		if (getMisappliedName() == null) {
			return;
		}
		String title = "";
		String author = "";
		TaxonName misappliedNameName = HibernateProxyHelper.deproxy(getMisappliedName().getName(), TaxonName.class);
		if (misappliedNameName.getAuthorshipCache() != null && !StringUtils.isBlank(misappliedNameName.getAuthorshipCache())&& getMisappliedName().getAppendedPhrase() == null){
			author = ", non " + misappliedNameName.getAuthorshipCache();
		}
		if (misapplication.getCitation() == null) {
		    if (getMisappliedName().getAppendedPhrase() != null ){
		        title = getMisappliedName().getAppendedPhrase();
		    }else{
		        title = "auct."; //$NON-NLS-1$
		        
		    }
		  
		} else {
			String sec = " sensu "; //$NON-NLS-1$
			if (getMisappliedName().getAppendedPhrase() != null ){
			    sec = getMisappliedName().getAppendedPhrase() + " "+sec; //$NON-NLS-1$
			}
			title += sec + misapplication.getCitation().getTitleCache();
			
			
			
			
		}
		title += author;
		title = title.replace("&", "&&"); //$NON-NLS-1$ //$NON-NLS-2$
		setNonEditableInfo(title, false);
	}

	/**
	 * <p>getMisappliedName</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 */
	public Taxon getMisappliedName() {
		return getTaxonBase();
	}

	/** {@inheritDoc} */
	@Override
	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.MISAPPLIEDNAME_FONT);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.container.AbstractGroupedContainer#showAnnotations()
	 */
	@Override
	public void showAnnotations() {
		if(getData().getSec() == null){
			getNameViewer().addAnnotation(
					new EditorAnnotation(0, Messages.MisapplicationContainer_SEC_REF_REQUIRED));
		}
		super.showAnnotations();
	}


	/** {@inheritDoc} */
	@Override
	protected void updateNonEditableInfo() {
		showSec();
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer#updateIndent()
	 */
	@Override
	protected void updateIndent() {
		setIndent(MISAPPLIEDNAME_INDENT);
	}
	
	/**
	 * <p>
	 * initTextViewer
	 * </p>
	 */
	@Override
	protected void initTextViewer() {

		// showNameRelations();

		updateIndent();

		updateIcon();

		String text = NameHelper.getDisplayNameCache(getData());

		if (text.length() == 0) {
			initEmptyText();
		} else {
			getNameViewer().setText(text);
			placeCursor();
		}
		calculateAnnotations();
	}
	
	@Override
    public void refresh() {
		// showNameRelations();

		String text = NameHelper.getDisplayNameCache(getData());

		if (getNameViewer().getTextWidget() == null) {
			// we might get here via dnd. Look slike it can be ignored
			return;
		}

		if (text.length() == 0) {
			initEmptyText();
		} else if (!getNameViewer().getTextWidget().getText().equals(text)) {
			removeListener();
			getNameViewer().getTextWidget().setText(text);
			addListener();
		}

		updateNonEditableInfo();

		updateIcon();
		// placeCursor();
		updateIndent();

		enableFreeText();
	}

}
