package eu.etaxonomy.taxeditor.editor.name.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.operation.MoveSynonymToAnotherAcceptedTaxonOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

public class MoveSynonymToAnotherAcceptedTaxonHandler extends AbstractHandler implements IPostOperationEnabled {
	private static final Logger logger = Logger
	.getLogger(MoveSynonymToAnotherAcceptedTaxonHandler.class);
	private MultiPageTaxonEditor editor;

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		editor =  (MultiPageTaxonEditor) HandlerUtil.getActiveEditor(event);
		Shell shell = HandlerUtil.getActiveShell(event);
		IEditorInput input = editor.getEditorInput();
		// Get synonym from selection before possible save dialog
		// because the selection will be null if save dialog is shown
		StructuredSelection selection = (StructuredSelection) HandlerUtil.getCurrentSelection(event);

		if (!EditorUtil.forceUserSave(editor, shell)) {
			return null;
		}

		if (!(input instanceof TaxonEditorInput)) {
			logger.error("Editor input is not TaxonEditorInput"); //$NON-NLS-1$
			return null;
		}

		if (selection==null || !(selection.getFirstElement() instanceof Synonym)) {
		    MessagingUtils.warningDialog(Messages.MoveSynonymToAnotherAcceptedTaxonHandler_NO_SELECTION, this, Messages.MoveSynonymToAnotherAcceptedTaxonHandler_NO_SELECTION_MESSAGE);
			logger.error("Selection does not contain a Synonym"); //$NON-NLS-1$
			return null;
		}

		Synonym synonym = (Synonym) selection.getFirstElement();
		Taxon oldParent = ((TaxonEditorInput)input).getTaxon();
		List<UUID> excludeTaxa = new ArrayList<UUID>();
		excludeTaxa.add(oldParent.getUuid());


		TaxonNode newParentNode = TaxonNodeSelectionDialog.select(HandlerUtil.getActiveShell(event), editor.getConversationHolder(), Messages.MoveSynonymToAnotherAcceptedTaxonHandler_SELECT_ACC_TAXON, excludeTaxa,  ((TaxonEditorInput)input).getTaxonNode(), ((TaxonEditorInput)input).getTaxonNode().getClassification());

		if(newParentNode!=null){
		   MoveSynonymToAnotherAcceptedTaxonOperation operation = new MoveSynonymToAnotherAcceptedTaxonOperation(Messages.MoveSynonymToAnotherAcceptedTaxonHandler_CHANGE_ACC_TAXON, EditorUtil.getUndoContext(),
		            oldParent.getUuid(), synonym.getUuid(), newParentNode.getTaxon(), editor);

		    AbstractUtility.executeOperation(operation);

		}
		return null;
	}

	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}



}
