/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>ChangeHomotypicalGroupBasionymOperation class.</p>
 *
 * @author p.ciardelli
 * @created 15.01.2009
 */
public class ChangeHomotypicalGroupBasionymOperation extends
		AbstractPostTaxonOperation {
	
	private final TaxonBase taxonBase;

	/**
	 * <p>Constructor for ChangeHomotypicalGroupBasionymOperation.</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param taxonBase a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public ChangeHomotypicalGroupBasionymOperation(String text,
			IUndoContext undoContext, Taxon taxon, TaxonBase taxonBase, IPostOperationEnabled postOperationEnabled) {
		super(text, undoContext, taxon, postOperationEnabled);

		this.taxonBase = taxonBase;
	}

	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		TaxonName name = taxonBase.getName();

		monitor.worked(20);
		name = HibernateProxyHelper.deproxy(name);
		HomotypicalGroup group = HibernateProxyHelper.deproxy(name.getHomotypicalGroup(), HomotypicalGroup.class);
        TaxonName oldBasionym = name.getBasionym();

		name.setAsGroupsBasionym();

		monitor.worked(40);
		oldBasionym = HibernateProxyHelper.deproxy(oldBasionym);
		if (oldBasionym != null){
			Set<TaxonBase> taxonBases = oldBasionym.getTaxonBases();
			if (!taxonBases.isEmpty()){
				TaxonBase oldBasionymTaxonBase = taxonBases.iterator().next();
				postExecute(oldBasionymTaxonBase);
			}
		}

		return postExecute(taxonBase);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO replace w method from cdmlib
		// FIXME this is also old code: reimplement
//		CdmUtil.setGroupBasionym(synonym.getName());

		return postExecute(taxonBase);
	}


}
