/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept;

import java.util.Map.Entry;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.key.AbstractGraphKeyEditor;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.model.LineSelection;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;
import eu.etaxonomy.taxeditor.view.detail.DetailsViewPart;
import eu.etaxonomy.taxeditor.view.supplementaldata.SupplementalDataViewPart;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author n.hoffmann
 * @created Jan 24, 2011
 * @version 1.0
 */
public class ConceptViewPart extends AbstractCdmEditorViewPart
implements IPartContentHasDetails, IPartContentHasSupplementalData{

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.concept"; //$NON-NLS-1$

	private ConceptRelationViewer viewer;

	@Override
	protected void selectionChanged_internal(IWorkbenchPart workbenchPart, ISelection selection) {
		if(workbenchPart == this){
		    return;
		}
        Object partObject = workbenchPart;
        Object wrappedPart = WorkbenchUtility.getE4WrappedPart(partObject);
        if(wrappedPart!=null){
            partObject = wrappedPart;
        }
		if(AbstractUtility.getActiveE4Editor() == null){
			showEmptyPage();
			return;
		}

        if (partObject instanceof DetailsViewPart || partObject instanceof SupplementalDataViewPart) {
            // do not show empty page as these views are also used to edit the
            // description selected in this view
            return;
        }
        else if(partObject instanceof BulkEditor && selection instanceof LineSelection){
			if(((LineSelection) selection).getFirstElement() instanceof Taxon){
				showViewer(partObject, (LineSelection) selection);
			}
			else{
				showEmptyPage();
			}
			return;
		}
		else if(partObject instanceof AbstractGraphKeyEditor){
			showEmptyPage();
			return;
		}
		else if(partObject instanceof MultiPageTaxonEditor){
			if(! partObject.equals(this.part)){
				IEditorInput input = ((IEditorPart) partObject).getEditorInput();
				showViewer(partObject, new StructuredSelection(input));
			}
		}
		else {
		    showEmptyPage();
		}

	}

	@Override
	protected String createPartTitle(Object selectedObject) {
		if(selectedObject instanceof TaxonEditorInput){
			return super.createPartTitle(((TaxonEditorInput) selectedObject).getTaxon());
		}
		return super.createPartTitle(selectedObject);
	}

	@Override
	public boolean onComplete() {
		return true;
	}

	@Override
	public Viewer getViewer() {
		return viewer;
	}

	@Override
	public void createViewer(Composite parent) {
		viewer = new ConceptRelationViewer(parent);

		viewer.setContentProvider(new ConceptContentProvider());
		viewer.setLabelProvider(new ConceptLabelProvider());


		viewer.setSorter(new ViewerSorter());

		getSite().setSelectionProvider(viewer);

		createMenu();
	}

	private void createMenu(){
		MenuManager menuManager = new MenuManager();
		menuManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

		getSite().registerContextMenu(menuManager, viewer);

		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);
		menuManager.setRemoveAllWhenShown(true);

		control.setMenu(menu);
	}

	@Override
	public void changed(Object object) {
		getViewer().refresh();
		super.changed(object);
	}

	private class ConceptRelationViewer extends ListViewer {

		public ConceptRelationViewer(Composite parent) {
			super(parent);
		}

		@Override
		public ISelection getSelection() {
			ISelection selection = super.getSelection();
			if(selection instanceof IStructuredSelection && !selection.isEmpty()){
				Object firstElement = ((IStructuredSelection) selection).getFirstElement();
				TaxonRelationship taxonRelationship = ((Entry<TaxonRelationship, Taxon>)firstElement).getKey();
				return new StructuredSelection(taxonRelationship);

			}
			return selection;
		}

		@Override
		public void setSelection(ISelection selection) {
			super.setSelection(selection);
		}

	}

	@Override
	protected String getViewName() {
		return Messages.ConceptViewPart_VIEWER_NAME;
	}

}
