/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.view.concept.operation.DeleteConceptRelationOperation;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * @author n.hoffmann
 * @created Jan 25, 2011
 * @version 1.0
 */
public class DeleteConceptRelationHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNameEditor editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(
				Page.NAME);
		
		ISelection selection = HandlerUtil.getActiveSite(event).getSelectionProvider().getSelection();
		
		Set<TaxonRelationship> relations = new HashSet<TaxonRelationship>();
		
		if(selection instanceof IStructuredSelection){
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			Iterator iterator = structuredSelection.iterator();
			
			while (iterator.hasNext()){
				Object element = iterator.next();
				if(element instanceof TaxonRelationship){
					relations.add((TaxonRelationship) element);
				}
			}
		}
		
		IWorkbenchPart activePart = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (activePart instanceof IPostOperationEnabled) ? (IPostOperationEnabled) activePart : null;
		
		try {
			AbstractPostOperation operation = new DeleteConceptRelationOperation(event.getCommand().getName(), 
					editor.getUndoContext(), editor.getTaxon(), relations, postOperationEnabled);
			EditorUtil.executeOperation(operation);
		} catch (NotDefinedException e) {
			MessagingUtils.warn(getClass(), "Command name not set"); //$NON-NLS-1$
		}
		
		return null;
	}
}
