/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.LineSelection;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * @author n.hoffmann
 * @created Jan 25, 2011
 * @version 1.0
 */
public class OpenRelatedConceptHandler extends AbstractHandler {

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		ISelection selection = HandlerUtil.getCurrentSelection(event);
        TaxonBase<?> relatedTaxon = null;

        if(selection instanceof IStructuredSelection){
            IStructuredSelection structuredSelection = (IStructuredSelection) selection;
            Object selectedElement = structuredSelection.getFirstElement();

            if(selectedElement instanceof TaxonRelationship){
                Object activeEditor = AbstractUtility.getActiveE4Editor();
                if(activeEditor instanceof MultiPageTaxonEditor){
                    relatedTaxon = getRelatedTaxon((TaxonRelationship) selectedElement, ((MultiPageTaxonEditor) activeEditor).getTaxon());
                }
                else if(activeEditor instanceof BulkEditor){
                    ISelection bulkSelection = ((BulkEditor)activeEditor).getSelectionProvider().getSelection();
                    if(bulkSelection instanceof LineSelection){
                        Object firstElement = ((LineSelection)bulkSelection).getFirstElement();
                        if(firstElement instanceof Taxon){
                            relatedTaxon = getRelatedTaxon((TaxonRelationship) selectedElement, (Taxon) firstElement);
                        }
                    }
                }
            }
        }

		if(relatedTaxon==null){
		    MessagingUtils.messageDialog(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN, getClass(),
		            String.format(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN_MESSAGE ,relatedTaxon), null);
		}
		else{
		    openConcept(relatedTaxon);
		}

		return null;
	}

    /**
     * @param relatedTaxon
     */
    protected void openConcept(TaxonBase<?> relatedTaxon) {
        try {
            EditorUtil.openTaxonBase(relatedTaxon.getUuid());
        } catch (PartInitException e) {
            MessagingUtils.messageDialog(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN, getClass(),
                    String.format(Messages.OpenRelatedConceptHandler_COULD_NOT_OPEN_MESSAGE ,relatedTaxon), e);
        }
    }

	private Taxon getRelatedTaxon(TaxonRelationship relationship, Taxon taxon){
		if (relationship.getFromTaxon().equals(taxon)){
			return relationship.getToTaxon();
		}
		else if(relationship.getToTaxon().equals(taxon)){
			return relationship.getFromTaxon();
		}
		else{
			return null;
		}
	}
}
