
package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.TreeNode;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeCacheStrategy;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivationEventType;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public abstract class AbstractAddDerivativeHandler {

	@Execute
	public void execute(@Named(IServiceConstants.ACTIVE_SELECTION) TreeNode node, @Named(IServiceConstants.ACTIVE_PART) MPart part) {
	    DerivateView derivateView = (DerivateView) part.getObject();

	    checkDirtyState(derivateView);

	    Object selectedDerivative = node.getValue();
	    if(isAllowed(selectedDerivative)){
	        Object createdElement = addElement(selectedDerivative);
	        refreshView(derivateView, createdElement);
	    }
	}

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) TreeNode node, MHandledMenuItem item) {
        boolean canExecute = false;

        Object derivate = node.getValue();
        canExecute = isAllowed(derivate);
        item.setVisible(canExecute);
        return canExecute;
    }

    private void refreshView(DerivateView derivateView, Object createdElement) {
        if(derivateView!=null){
            derivateView.getConversationHolder().commit();
            derivateView.refreshTree(createdElement);
        }
    }

	private void checkDirtyState(DerivateView derivateView){
	        if(derivateView!=null){
	            if(derivateView.isDirty()){
	                MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
	                return;
	            }
	        }
	}

	protected Object addElement(Object selectedDerivative){
	    Object createdElement = null;
	    SpecimenOrObservationBase<?> specimenOrObservationBase = (SpecimenOrObservationBase<?>) selectedDerivative;
	    DerivedUnit derivedUnit = createDerivative();
	    specimenOrObservationBase.addDerivationEvent(DerivationEvent.NewSimpleInstance(specimenOrObservationBase, derivedUnit, getDerivationeType()));
	    derivedUnit.setCacheStrategy(new DerivedUnitFacadeCacheStrategy());
	    derivedUnit.getTitleCache(); //update title cache
	    createdElement = derivedUnit;
	    CdmStore.getService(IOccurrenceService.class).merge((SpecimenOrObservationBase<?>) selectedDerivative, true);
	    return createdElement;
	}

	protected abstract DerivedUnit createDerivative();

	protected abstract DerivationEventType getDerivationeType();

    protected abstract boolean isAllowed(Object derivative);

}