
package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.AppModelId;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class CreateFieldUnitHandler {

	@Execute
	public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart part, MHandledMenuItem item) {
        DerivateView derivateView = (DerivateView) part.getObject();
        if(item.getElementId().equals(AppModelId.HANDLEDMENUITEM_EU_ETAXONOMY_TAXEDITOR_EDITOR_HANDLEDMENUITEM_CREATE_FIELD_UNIT_FOR_TAXON)){
            Object selectionInput = derivateView.getSelectionInput();
            if(selectionInput instanceof CdmBase && ((CdmBase) selectionInput).isInstanceOf(Taxon.class)){
                final Taxon taxon = HibernateProxyHelper.deproxy(CdmStore.getService(ITaxonService.class).load(((CdmBase)selectionInput).getUuid()), Taxon.class);
                if(derivateView.isDirty()){
                    MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                    return;
                }

                FieldUnit fieldUnit = FieldUnit.NewInstance();
                fieldUnit = CdmBase.deproxy(CdmStore.getService(IOccurrenceService.class).save(fieldUnit), FieldUnit.class);

                IndividualsAssociation association = IndividualsAssociation.NewInstance(fieldUnit);
                TaxonDescription description;
                if(!taxon.getDescriptions().isEmpty()){
                    description = taxon.getDescriptions().iterator().next();
                }
                else{
                    description = TaxonDescription.NewInstance(taxon);
                }
                description.addElement(association);
                CdmStore.getService(ITaxonService.class).merge(taxon);


                derivateView.getConversationHolder().commit();
                derivateView.addFieldUnit(fieldUnit);
                derivateView.updateRootEntities();
                derivateView.refreshTree(fieldUnit);
            }
        }
        else{
            if(derivateView.isDirty()){
                MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                return;
            }
            FieldUnit fieldUnit = FieldUnit.NewInstance();
            fieldUnit = CdmBase.deproxy(CdmStore.getService(IOccurrenceService.class).save(fieldUnit), FieldUnit.class);

            derivateView.getConversationHolder().commit();
            derivateView.addFieldUnit(fieldUnit);
            derivateView.updateRootEntities();
            derivateView.refreshTree(fieldUnit);
        }
	}


	@CanExecute
	public boolean canExecute(@Named(IServiceConstants.ACTIVE_PART) MPart part, MHandledMenuItem item) {
	    DerivateView derivateView = (DerivateView) part.getObject();
	    boolean listenToSelectionChange = derivateView.isListenToSelectionChange();
	    setItemVisibility(item, listenToSelectionChange);
	    if(listenToSelectionChange &&
	            item.getElementId().equals(AppModelId.HANDLEDMENUITEM_EU_ETAXONOMY_TAXEDITOR_EDITOR_HANDLEDMENUITEM_CREATE_FIELD_UNIT_FOR_TAXON)){
	        Object selectionInput = derivateView.getSelectionInput();
	        if(selectionInput instanceof CdmBase && ((CdmBase) selectionInput).isInstanceOf(Taxon.class)){
	            final Taxon taxon = HibernateProxyHelper.deproxy(CdmStore.getService(ITaxonService.class).load(((CdmBase)selectionInput).getUuid()), Taxon.class);
	            item.setLabel(String.format(Messages.CreateFieldUnitContextMenu_CREATE_FIELD_UNIT_FOR, taxon.getName()));
	        }
	    }
	    return true;
	}


    private void setItemVisibility(MHandledMenuItem item, boolean isListeningToSelectionChange) {
        if(item.getElementId().equals(AppModelId.HANDLEDMENUITEM_EU_ETAXONOMY_TAXEDITOR_EDITOR_HANDLEDMENUITEM_CREATEFIELDUNIT)){
            item.setVisible(!isListeningToSelectionChange);
        }
        else{
            item.setVisible(isListeningToSelectionChange);
        }
    }

}