package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.AppModelId;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.DefaultOpenHandlerBase;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

public class OpenDerivativeEditorForTaxonNode extends DefaultOpenHandlerBase<TaxonNode> {

    protected static final String OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN = Messages.OpenDerivativeEditorForTaxonNode_COULD_NOT_OPEN;

    @Override
    protected TaxonNode getEntity(UUID uuid) {
        return CdmStore.getService(ITaxonNodeService.class).load(uuid);
    }

    @Override
    protected void open(ExecutionEvent event, TaxonNode entity) {
        FindOccurrencesConfigurator config = new FindOccurrencesConfigurator();
        UUID taxonUuid = entity.getTaxon().getUuid();
        config.setAssociatedTaxonUuid(taxonUuid);
        List<SpecimenOrObservationBase> specimens = CdmStore.getService(IOccurrenceService.class).findByTitle(config).getRecords();
        Collection<UUID> specimenUuids = new ArrayList<UUID>();
        for (SpecimenOrObservationBase specimenOrObservationBase : specimens) {
            specimenUuids.add(specimenOrObservationBase.getUuid());
        }
        DerivateViewEditorInput input = new DerivateViewEditorInput(specimenUuids, taxonUuid);
        MPart part = TaxeditorStorePlugin.getDefault().getWorkbench().getService(EPartService.class).showPart(
                AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_VIEW_DERIVATE_DERIVATEVIEW, PartState.ACTIVATE);
        DerivateView derivateView = (DerivateView) part.getObject();
        derivateView.init(input);
    }

}
