package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.TreeNode;

import eu.etaxonomy.cdm.api.service.molecular.ISequenceService;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class SingleReadRemoveHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart part, @Named(IServiceConstants.ACTIVE_SELECTION) TreeNode selectedTreeNode){
        DerivateView derivateView = (DerivateView)part.getObject();
        if(derivateView.isDirty()){
            MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
            return;
        }
        if(selectedTreeNode!=null && selectedTreeNode.getParent()!=null &&
                selectedTreeNode.getValue() instanceof SingleRead && selectedTreeNode.getParent().getValue() instanceof Sequence) {
            Sequence sequence = (Sequence) selectedTreeNode.getParent().getValue();
            sequence.removeSingleRead((SingleRead) selectedTreeNode.getValue());
            CdmStore.getService(ISequenceService.class).merge(sequence);

            derivateView.getConversationHolder().commit();
            derivateView.refreshTree();
        }
        derivateView.updateLabelCache();
        derivateView.refreshTree();
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_PART) MPart part,
            @Optional @Named(IServiceConstants.ACTIVE_SELECTION) TreeNode selectedTreeNode,
            MHandledMenuItem menuItem) {
        boolean canExecute = false;
        if(selectedTreeNode!=null){
            Object value = selectedTreeNode.getValue();
            canExecute = value instanceof SingleRead && ((DerivateView) part.getObject()).getMultiLinkSingleReads().contains(value);
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
