/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.derivate.searchFilter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.ResourceManager;

import eu.etaxonomy.taxeditor.editor.l10n.Messages;

/**
 * The widgets of the {@link DerivateSearchView}<br>
 *
 */
public class DerivateSearchComposite extends Composite {

    public static final int ALL_SPECIMENS = 0;
    public static final int ASSIGNED_SPECIMENS = 1;
    public static final int UNASSIGNED_SPECIMENS = 2;

    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());
    private final Text searchField;
    private final Combo comboDerivateType;
    private final Button buttonSearch;
    private final Text textTaxonName;
    private final Button btnBrowseTaxa;
    private final Label lblTaxon;
    private final Label lblDerivateType;
    private final Button btnClearTaxon;
    private Label lbltaxonAssignment;
    private Combo comboTaxonAssignment;
    private Label lblTitleCache;

    /**
     * Create the composite.
     *
     * @param parent
     * @param style
     */
    public DerivateSearchComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(7, false));

        lblTaxon = new Label(this, SWT.NONE);
        lblTaxon.setText(Messages.DerivateSearchComposite_TAXON);

        textTaxonName = formToolkit.createText(this, Messages.DerivateSearchComposite_NEW_TEXT, SWT.BORDER);
        textTaxonName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        textTaxonName.setEnabled(false);
        textTaxonName.setText(""); //$NON-NLS-1$

        btnBrowseTaxa = formToolkit.createButton(this, "", SWT.NONE); //$NON-NLS-1$
        btnBrowseTaxa.setAlignment(SWT.RIGHT);
        btnBrowseTaxa.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/prj_obj.gif")); //$NON-NLS-1$ //$NON-NLS-2$

        btnClearTaxon = formToolkit.createButton(this, "", SWT.NONE); //$NON-NLS-1$
        btnClearTaxon.setAlignment(SWT.RIGHT);
        btnClearTaxon.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
        btnClearTaxon.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif")); //$NON-NLS-1$ //$NON-NLS-2$

        lbltaxonAssignment = new Label(this, SWT.NONE);
        lbltaxonAssignment.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lbltaxonAssignment.setText(Messages.DerivateSearchComposite_TAXON_ASSIGNMENT);

        comboTaxonAssignment = new Combo(this, SWT.NONE);
        comboTaxonAssignment.setItems(new String[] { Messages.DerivateSearchComposite_ALL, Messages.DerivateSearchComposite_YES, Messages.DerivateSearchComposite_NO });
        comboTaxonAssignment.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(comboTaxonAssignment);
        formToolkit.paintBordersFor(comboTaxonAssignment);
        comboTaxonAssignment.select(ALL_SPECIMENS);
        new Label(this, SWT.NONE);

        lblTitleCache = new Label(this, SWT.NONE);
        lblTitleCache.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblTitleCache.setText(Messages.DerivateSearchComposite_TITLE_CACHE);

        searchField = formToolkit.createText(this, "New Text", SWT.BORDER); //$NON-NLS-1$
        searchField.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 3, 1));
        searchField.setText(""); //$NON-NLS-1$

        lblDerivateType = new Label(this, SWT.NULL);
        lblDerivateType.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblDerivateType.setText(Messages.DerivateSearchComposite_DERIVATE_TYPE);

        comboDerivateType = new Combo(this, SWT.READ_ONLY);
        comboDerivateType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        formToolkit.paintBordersFor(comboDerivateType);

        buttonSearch = new Button(this, SWT.NONE);
        formToolkit.adapt(buttonSearch, true, true);
        buttonSearch.setText(Messages.DerivateSearchComposite_SEARCH);

    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }

    public Text getSearchField() {
        return searchField;
    }

    public Combo getComboDerivateType() {
        return comboDerivateType;
    }

    public Button getButtonSearch() {
        return buttonSearch;
    }

    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        searchField.setEnabled(enabled);
        comboDerivateType.setEnabled(enabled);
        comboTaxonAssignment.setEnabled(enabled);
        buttonSearch.setEnabled(enabled);
        btnBrowseTaxa.setEnabled(enabled);
        btnClearTaxon.setEnabled(enabled);
        lblTaxon.setEnabled(enabled);
        lblTitleCache.setEnabled(enabled);
        lbltaxonAssignment.setEnabled(enabled);
        lblDerivateType.setEnabled(enabled);
    }


    public Button getBtnBrowseTaxa() {
        return btnBrowseTaxa;
    }

    public Text getTextTaxonName() {
        return textTaxonName;
    }

    public Button getBtnClearTaxon() {
        return btnClearTaxon;
    }
    public Combo getComboTaxonAssignment() {
        return comboTaxonAssignment;
    }

    public Label getLbltaxonAssignment() {
        return lbltaxonAssignment;
    }
}
