/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureNode;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveViewPart;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateDescriptionElementOperation;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author n.hoffmann
 * @created 17.04.2009
 * @version 1.0
 */
public class DynamicFeatureMenu extends CompoundContributionItem {

	/** {@inheritDoc} */
	@Override
	protected IContributionItem[] getContributionItems() {

		return new IContributionItem[] { new ContributionItem() {
			@Override
            public void fill(Menu menu, int index) {
			    ISelectionService selectionService = TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getSelectionService();

				ISelection selection = selectionService.getSelection(DescriptiveViewPart.ID);

				if (selection instanceof IStructuredSelection) {
					IStructuredSelection structuredSelection = (IStructuredSelection) selection;
					Object selectedElement = structuredSelection
							.getFirstElement();

					if (selectedElement instanceof DescriptionBase<?>) {
						FeatureTree featureTree = getFeatureTree((DescriptionBase<?>) selectedElement);

						for (FeatureNode childNode : featureTree.getRootChildren()) {
							createMenuItem(menu, childNode.getFeature());

						}
					} else if (selectedElement instanceof FeatureNodeContainer) {
						FeatureNode featureNode = ((FeatureNodeContainer) selectedElement)
								.getFeatureNode();

						// add the feature to the menu
						createMenuItem(menu, featureNode.getFeature());

						// add possible children to the menu
						for (FeatureNode childNode : featureNode.getChildNodes()) {
							createMenuItem(menu, childNode.getFeature());
						}
					} else if (selectedElement instanceof DescriptionElementBase) {
						Feature feature = ((DescriptionElementBase) selectedElement)
								.getFeature();
						createMenuItem(menu, feature);
					}
				}
			}
		} };
	}

	private void createMenuItem(Menu menu, final Feature feature) {
		MenuItem menuItem = new MenuItem(menu, -1);
		final Feature deproxiedFeature = HibernateProxyHelper.deproxy(feature, Feature.class);

		String label = deproxiedFeature.getLabel(PreferencesUtil.getGlobalLanguage());
		if (label == null){
			menuItem.setText(deproxiedFeature.getLabel());
		}else{
			menuItem.setText(label);
		}
		menuItem.addSelectionListener(new SelectionListener() {

			@Override
            public void widgetDefaultSelected(SelectionEvent e) {
			}

			@Override
            public void widgetSelected(SelectionEvent ev) {
				Event event = new Event();
				event.data = deproxiedFeature;
				try {
				    TaxeditorEditorPlugin.getDefault().getWorkbench().getService(IHandlerService.class).executeCommand(
							CreateDescriptionElementOperation.ID, event);
				} catch (Exception e) {
					MessagingUtils.error(getClass(), e);
				}
			}
		});

	}

	/**
	 * Retrieves the feature tree associated with the given description
	 *
	 * TODO as of now this is always the same thing because feature trees may
	 * not be associated to descriptions yet.
	 *
	 * @param description
	 * @return
	 */
	private FeatureTree getFeatureTree(DescriptionBase description) {
		FeatureTree featureTree = null;

		// TODO change this to the feature tree associated with this taxon
		// description
		if (description.hasStructuredData()) {
			featureTree = PreferencesUtil
					.getDefaultFeatureTreeForStructuredDescription();
		} else {
			featureTree = PreferencesUtil
					.getDefaultFeatureTreeForTextualDescription();
		}

		if (featureTree == null) {
			featureTree = FeatureTree.NewInstance(CdmStore.getTermManager()
					.getPreferredTerms(Feature.class));
		}

		return featureTree;
	}
}
