/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteTaxonDescriptionOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 05.02.2009
 * @version 1.0
 */
public class DeleteTaxonDescriptionOperation extends AbstractPostTaxonOperation {

	private final TaxonDescription description;

	/**
	 * <p>Constructor for DeleteTaxonDescriptionOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param description a {@link eu.etaxonomy.cdm.model.description.TaxonDescription} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public DeleteTaxonDescriptionOperation(String label,
	        IUndoContext undoContext,
			TaxonDescription description,
			IPostOperationEnabled postOperationEnabled,
			ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
		super(label, undoContext, postOperationEnabled, cdmEntitySessionEnabled);

		this.description = description;
		element = description.getTaxon();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		monitor.worked(20);
		if (description != null){
		    element.removeDescription(description);
		    CdmStore.getService(IDescriptionService.class).deleteDescription(description.getUuid());
			return postExecute(description);
		}
		return null;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

	    element.addDescription(description);

		return postExecute(null);
	}
}
