/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.internal.E4PartWrapper;

import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveViewPart;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasMedia;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;
import eu.etaxonomy.taxeditor.view.detail.DetailsViewPart;
import eu.etaxonomy.taxeditor.view.supplementaldata.SupplementalDataViewPart;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author n.hoffmann
 * @created Jun 15, 2010
 * @version 1.0
 */
public class MediaViewPart extends AbstractCdmEditorViewPart implements IPartContentHasDetails, IPartContentHasSupplementalData {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.media"; //$NON-NLS-1$


	private TreeViewer viewer;

	/** {@inheritDoc} */
	@Override
	public void createViewer(Composite parent) {

		viewer = new TreeViewer(new Tree(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION));

		viewer.setContentProvider(new MediaContentProvider());
		viewer.setLabelProvider(new MediaLabelProvider());
		viewer.setAutoExpandLevel(2);

		// Propagate selection from viewer
		getSite().setSelectionProvider(viewer);

		// Add context menu to tree
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		menuMgr.setRemoveAllWhenShown(true);
		getSite().registerContextMenu(menuMgr, viewer);

		Control control = viewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		menuMgr.setRemoveAllWhenShown(true);
		control.setMenu(menu);

	}

	/** {@inheritDoc} */
	@Override
	protected ISelection getInitialSelection() {
		if(getEditor() != null){
			return new StructuredSelection(getEditor().getEditorInput());
		}

		return super.getInitialSelection();
	}

	/** {@inheritDoc} */
	@Override
    protected void selectionChanged_internal(IWorkbenchPart workbenchPart, ISelection selection) {
	    Object partObject = workbenchPart;

	    if(partObject==this){
	        return;
	    }

        Object wrappedPart = WorkbenchUtility.getE4WrappedPart(partObject);
        if(wrappedPart!=null){
            partObject = wrappedPart;
        }
//		if(AbstractUtility.getActiveE4Editor() == null){
//			showEmptyPage();
//			return;
//		}


        if (partObject instanceof DetailsViewPart || partObject instanceof SupplementalDataViewPart
                || partObject instanceof DescriptiveViewPart) {
            // do not show empty page as these views are also used to edit the
            // description selected in this view
            return;
        }

		if(partObject instanceof IPartContentHasMedia && ((IPartContentHasMedia) partObject).canAttachMedia() ){
		    if(selection instanceof IStructuredSelection){
		    	Object firstElement = ((IStructuredSelection) selection).getFirstElement();
		    	if (partObject instanceof MultiPageTaxonEditor){
	            	if (firstElement != ((MultiPageTaxonEditor)partObject).getTaxon()){
	            		showEmptyPage();
	            		return;
	            	}
	            }
		        if(firstElement instanceof TreeNode){
		            showViewer(partObject, new StructuredSelection(((TreeNode) firstElement).getValue()));
		            return;
		        }
		        else if(firstElement!=null
	                    && firstElement instanceof IDescribable<?>){
		            showViewer(partObject, (IStructuredSelection) selection);
		            return;
		        }
		    }
		}
		if (selection instanceof IStructuredSelection){
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			if(structuredSelection.getFirstElement() instanceof TreeNode){
					if (((TreeNode)structuredSelection.getFirstElement()).getValue() instanceof SpecimenOrObservationBase){
				    structuredSelection = new StructuredSelection(((TreeNode)structuredSelection.getFirstElement()).getValue());
				    showViewer(partObject, structuredSelection);
				    return;
				}
			}
		}

		showEmptyPage();
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object object) {
	    viewer.expandToLevel(2);
	    viewer.refresh();
	    if(object != null){
	        StructuredSelection selection = new StructuredSelection(object);
	        viewer.setSelection(selection, true);
	    }
	    //TODO: should be replaced with the possibility to set views dirty
	    // when we move to Eclipse 4
	    // take a look at ISaveblePart
	    if(part instanceof BulkEditor && !(object instanceof SpecimenOrObservationBase<?>)){
	        ((BulkEditor) part).forceDirty();
	        IStructuredSelection selection = (IStructuredSelection) ((BulkEditor) part).getSelectionProvider().getSelection();
            ((BulkEditor) part).changed(selection.getFirstElement());

	    }
	    if (part instanceof E4PartWrapper){
	    	part = WorkbenchUtility.getE4WrappedPart(part);
	    	if (part instanceof IDirtyMarkable){
	    		 StructuredSelection selection = new StructuredSelection(object);
	    		((IDirtyMarkable)part).changed(selection.getFirstElement());
	    	}
	    }


	    super.changed(object);
	}

	/** {@inheritDoc} */
	@Override
	public Viewer getViewer() {
		return viewer;
	}

	@Override
    public boolean onComplete() {
		return true;
	}

	@Override
	protected String getViewName() {
		return Messages.MediaViewPart_MEDIA;
	}
}
