/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.AddDerivedUnitFacadeMediaOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.media.MediaViewPart;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * @author p.ciardelli
 * @version $Id: $
 */
public class AddImageGalleryHandler extends AbstractHandler {

	/** {@inheritDoc} */
	@Override
    public Object execute(ExecutionEvent event) throws ExecutionException {

	    IWorkbenchPart part = HandlerUtil.getActivePart(event);
	    IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;

	    if(part instanceof MediaViewPart){
	        Object viewerInput = ((MediaViewPart)part).getViewer().getInput();
	        try {
	            if(viewerInput instanceof CdmBase){
	                if(((CdmBase) viewerInput).isInstanceOf(Taxon.class)){
	                    Taxon taxon = HibernateProxyHelper.deproxy(viewerInput, Taxon.class);

	                    AbstractPostOperation<?> operation = new CreateTaxonDescriptionOperation(event.getCommand().getName(),
	                            EditorUtil.getUndoContext(), taxon, postOperationEnabled, true);

	                    AbstractUtility.executeOperation(operation);
	                }
	                else if(((CdmBase) viewerInput).isInstanceOf(SpecimenOrObservationBase.class)){
	                    SpecimenOrObservationBase<?> specimen = HibernateProxyHelper.deproxy(viewerInput, SpecimenOrObservationBase.class);

                        AbstractPostOperation<?> operation = new AddDerivedUnitFacadeMediaOperation(event.getCommand().getName(),
	                            BulkEditorUtil.getUndoContext(), specimen, postOperationEnabled);

	                    AbstractUtility.executeOperation(operation);
	                }
	            }
	        } catch (NotDefinedException e) {
	            MessagingUtils.warn(getClass(), "Command name not set."); //$NON-NLS-1$
	        }
	    }
	    return null;
	}

}
