/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.uses;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.StyledString;

import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.UsageTermCollection;

/**
 * UsesLabelProvider Class
 * @author a.theys
 * @created mar 13, 2012
 * @version 1.0
 */
public class UsesLabelProvider extends ColumnLabelProvider implements
		IStyledLabelProvider {

    private static final String USE = Messages.UsesLabelProvider_USE;
    private static final String NO_DATA_PROVIDED = Messages.UsesLabelProvider_NO_DATA;
    private static final String SEMICOLON = ";"; //$NON-NLS-1$

    /** {@inheritDoc} */
	@Override
    public String getText(Object element) {
		String text = ""; //$NON-NLS-1$

		if (element instanceof DescriptionBase) {
			text = ((DescriptionBase) element).getTitleCache();
			if (text == null || text.length() == 0) {
				text = USE+Messages.UsesLabelProvider_NO_LABEL;
			}
			else {
				text = USE + text;
			}
		}
		else if (element instanceof CategoricalData) {
			if (!((CategoricalData) element).getStateData().isEmpty()) {
				boolean isUseCategoryAbsent = true;
				for (StateData statedata : ((CategoricalData) element).getStateData()) {
					if(statedata.getState() != null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()) !=null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useCategoryVocabularyLabel)) {
						text = text + statedata.getState().getTitleCache() + SEMICOLON;
						isUseCategoryAbsent = false;
					}
				}
				if(isUseCategoryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}

				boolean isUseSubCategoryAbsent = true;
				for (StateData statedata : ((CategoricalData) element).getStateData()) {
					if(statedata.getState() != null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()) !=null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useSubCategoryVocabularyLabel)) {
						text = text + statedata.getState().getTitleCache() + SEMICOLON;
						isUseSubCategoryAbsent = false;
					}
				}
				if(isUseSubCategoryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}
			}
			else {
				text = text + UsageTermCollection.notAvailableLabel + SEMICOLON + UsageTermCollection.notAvailableLabel + SEMICOLON;
			}
			if (!((CategoricalData) element).getModifiers().isEmpty()) {
				for (DefinedTerm modifier: ((CategoricalData) element).getModifiers()){
					modifier.getPartOf();
					modifier.getVocabulary();
				}

				boolean isPlantPartAbsent = true;
				for (DefinedTerm modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.plantPartLabel)) {
						text = text +  modifier.getTitleCache()+ SEMICOLON;
						isPlantPartAbsent = false;
					}

				}
				if (isPlantPartAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}

				boolean isHumanGroupAbsent = true;
				for (DefinedTerm modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.humanGroupLabel)) {
						text = text +  modifier.getTitleCache()+ SEMICOLON;
						isHumanGroupAbsent = false;
					}
				}
				if (isHumanGroupAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}

				boolean isEthnicGroupAbsent = true;
				for (DefinedTerm modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.ethnicGroupLabel)) {
						text = text +  modifier.getTitleCache()+ SEMICOLON;
						isEthnicGroupAbsent = false;
					}
				}
				if (isEthnicGroupAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}

				boolean isCountryAbsent = true;
				for (DefinedTerm modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.countryLabel)) {
						text = text +  modifier.getTitleCache()+ SEMICOLON;
						isCountryAbsent = false;
					}
				}
				if (isCountryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + SEMICOLON;
				}
			}
			else {
				text = text + UsageTermCollection.notAvailableLabel + SEMICOLON + UsageTermCollection.notAvailableLabel + SEMICOLON + UsageTermCollection.notAvailableLabel + SEMICOLON + UsageTermCollection.notAvailableLabel + SEMICOLON;
			}

			if (text == null || text.length() == 0) {
				text = NO_DATA_PROVIDED;
			}
			else {
				String[] textElements = text.split(SEMICOLON);
				String dataportalString = textElements[0] + SEMICOLON + textElements[1] + SEMICOLON + textElements[5] + SEMICOLON + textElements[2] + SEMICOLON + textElements[3] + SEMICOLON + textElements[4] + SEMICOLON ;

				((CategoricalData)element).putModifyingText(CdmStore.getDefaultLanguage(), dataportalString);
			}
			return text;
		}

		else if (element instanceof TextData) {
			List<Language> languages = Arrays.asList(new Language[]{CdmStore.getDefaultLanguage()});
			LanguageString languageString = ((TextData) element).getPreferredLanguageString(languages);
			text = languageString != null ? languageString.getText() : null;
			if (text == null || text.length() == 0) {
				text = NO_DATA_PROVIDED;
			}
		}

		else if (element instanceof FeatureNodeContainer){
			text = ((FeatureNodeContainer)element).getFeature().getTitleCache();
		}

		return  text;
	}

	/** Function dertmining which vocabulary is the term part of
	 *
	 * @param term
	 * @param vocabularyExpected
	 * @return
	 */
	private boolean GetVocabularyType(DefinedTerm term, String vocabularyExpected) {
		if ((term.getPartOf() != null)  && (term.getPartOf().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((term.getVocabulary() != null) && (term.getPartOf() == null) && (term.getVocabulary().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((vocabularyExpected.equals(UsageTermCollection.ethnicGroupLabel))  && ((term.getPartOf() != null))) {
			return true;
		}
	return false;
	}

	/** {@inheritDoc} */
	@Override
	public StyledString getStyledText(Object element) {
		return new StyledString(this.getText(element),
				StyledString.QUALIFIER_STYLER);
	}



}
