/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.workingSet;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DragSourceEvent;

import eu.etaxonomy.cdm.model.description.FeatureNode;
import eu.etaxonomy.taxeditor.featuretree.FeatureNodeDragListener;
import eu.etaxonomy.taxeditor.featuretree.FeatureNodeTransfer;

/**
 * @author pplitzner
 * @since Jun 2, 2017
 *
 */
public class CharacterDragListener extends FeatureNodeDragListener {

    private TreeViewer structureViewer;
    private TreeViewer propertyViewer;

    /**
     * @param propertyViewer
     */
    public CharacterDragListener(TreeViewer structureViewer, TreeViewer propertyViewer) {
        super(propertyViewer);
        this.structureViewer = structureViewer;
        this.propertyViewer = propertyViewer;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void dragSetData(DragSourceEvent event) {
        FeatureNode structure = (FeatureNode) ((IStructuredSelection) structureViewer.getSelection()).getFirstElement();
        FeatureNode property = (FeatureNode) ((IStructuredSelection) propertyViewer.getSelection()).getFirstElement();
        FeatureNode[] character = new FeatureNode[2];
        character[0] = structure;
        character[1] = property;
        if (FeatureNodeTransfer.getInstance().isSupportedType(event.dataType)) {
            event.data = character;
        }
    }

}
