/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.workingSet;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.description.FeatureNode;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.taxeditor.featuretree.FeatureNodeTransfer;

/**
 *
 * @author pplitzner
 * @date Jun 1, 2017
 *
 */
public class CharacterDropAdapter extends ViewerDropAdapter {

	protected CharacterDropAdapter(Viewer viewer) {
	    super(viewer);
	}

	@Override
	public boolean performDrop(Object data) {

	    FeatureNode target = (FeatureNode) getCurrentTarget();
		int currentLocation = getCurrentLocation();
		Object[] characterData = (Object[]) data;

		//create new Character
		if(characterData.length==2){
		    FeatureNode structureNode = (FeatureNode) characterData[0];
		    FeatureNode propertyNode = (FeatureNode) characterData[1];
		    String label = structureNode.getFeature().toString()+" "+propertyNode.getFeature().toString();
		    Character character = Character.NewInstance(structureNode, propertyNode, null, label, label);
		    FeatureNode characterNode = FeatureNode.NewInstance(character);

		    if(getViewer().getInput()==null){
		        //create new feature tree
		        FeatureTree featureTree = FeatureTree.NewInstance();
		        getViewer().setInput(featureTree);
		    }
            if(target==null){
                FeatureNode root = ((FeatureTree)getViewer().getInput()).getRoot();
                root.addChild(characterNode);
            }
            else{
                target.addChild(characterNode);
            }
		}
		getViewer().refresh();
		return true;
	}

	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferType) {
		boolean transferDataIsSupported = FeatureNodeTransfer.getInstance().isSupportedType(transferType);
		return transferDataIsSupported;
	}

}
